import requests
import base64
import json

class API():
    def __init__(self, api_username:str , api_password:str):
        credentials = f"{api_username}:{api_password}"
        credentials = base64.b64encode(credentials.encode('utf-8')).decode('utf-8')
        self.auth_token = f"Basic {credentials}"

    def get_service_balance(self):
        """
        Queries the service account wallet balance.
        """
        try:
            url ="https://backend.payhero.co.ke/api/v2/wallets?wallet_type=service_wallet"
            headers ={ "Authorization": self.auth_token}
            response= requests.get(url, headers=headers)
            if response.status_code ==200:
                return response.json()
        except Exception as error:
            print(error)

    def get_payment_balance(self):
        """
        Queries the payment wallet balance.
        """
        url = 'https://backend.payhero.co.ke/api/v2/wallets?wallet_type=payment_wallet'
        headers ={ "Authorization": self.auth_token}
        try:
            response = requests.get(url, headers=headers)
            if response.status_code == 200:
                return response.json()
        except Exception as error:
            print(error)
        

    def topup_service_wallet(self, phone_number, amount: int):
        """
        Initiates a top-up for the service wallet on Pay Hero.
        """
        url = 'https://backend.payhero.co.ke/api/v2/topup'
        headers ={ "Authorization": self.auth_token}
        data ={
            "amount": amount,
            "phone_number":phone_number
        }
        try:
            response = requests.post(url, headers=headers, data= json.dumps(data) )
            if response.status_code ==201:
                return response.json()
            else:
                print( response.json() )
        except Exception as error:
            print(error)
        

    def initiate_mpesa_stk_push(self, amount:int , phone_number:str, channel_id:str, provider:str, external_reference:str, customer_name:str, callback_url:str):
        """
        Initiates an MPESA STK Push request to a customer's phone.
        """
        url = 'https://backend.payhero.co.ke/api/v2/payments'
        headers = {
            'Content-Type': 'application/json',
            "Authorization": self.auth_token
        }
        data = {
            "amount": amount,
            "phone_number": phone_number,
            "channel_id": channel_id,
            "provider": provider,
            "external_reference": external_reference,
            "customer_name": customer_name,
            "callback_url": callback_url
        }
        try:
            response = requests.post(url , headers=headers, data=json.dumps(data) )
            if response.status_code == 201:
                return response.json()
        except Exception as error:
            print(error)


    def fetch_transactions(self,page:int , per:int ):
        """
        Fetches the account transactions.

        """
        url = f'https://backend.payhero.co.ke/api/v2/transactions?page={page}&per={per}'
        headers ={ "Authorization": self.auth_token}
        try:
            response = requests.get(url, headers=headers)
            if response.status_code == 200:
                return response.json()
        except Exception as error:
            print(error)
    
    def fetch_transaction_status(self, reference:str):
        """
        Fetches the account transactions.
        """
        url = 'https://backend.payhero.co.ke/api/v2/transaction-status?reference={reference}'
        headers ={ "Authorization": self.auth_token}
        try:
            response = requests.get(url, headers=headers)
            if response.status_code == 200:
                return response.json()
        except Exception as error:
            print(error)



    



import pytest
from unittest.mock import patch
from PayHero.main import API  # Adjust the import based on your package structure
from dotenv import load_dotenv
from os import environ
load_dotenv() 


@pytest.fixture
def api_instance():
    """Fixture to create an API instance for testing."""
    api_username = environ.get('TESTS_USERNAME')
    api_password = environ.get('TESTS_PASSWORD')
    return API(api_username, api_password)

def test_get_service_balance(api_instance, mocker):
    """Test the get_service_balance method."""
    mock_response = {
        "balance": 100.0
    }
    mocker.patch('requests.get', return_value=mocker.Mock(status_code=200, json=lambda: mock_response))

    response = api_instance.get_service_balance()
    assert response == mock_response

def test_get_payment_balance(api_instance, mocker):
    """Test the get_payment_balance method."""
    mock_response = {
        "balance": 50.0
    }
    mocker.patch('requests.get', return_value=mocker.Mock(status_code=200, json=lambda: mock_response))

    response = api_instance.get_payment_balance()
    assert response == mock_response

def test_topup_service_wallet(api_instance, mocker):
    """Test the topup_service_wallet method."""
    mock_response = {
        "status": "success",
        "message": "Top-up successful"
    }
    mocker.patch('requests.post', return_value=mocker.Mock(status_code=201, json=lambda: mock_response))

    response = api_instance.topup_service_wallet('254700000000', 100)
    assert response == mock_response

def test_initiate_mpesa_stk_push(api_instance, mocker):
    """Test the initiate_mpesa_stk_push method."""
    mock_response = {
        "status": "success",
        "message": "STK Push initiated"
    }
    mocker.patch('requests.post', return_value=mocker.Mock(status_code=201, json=lambda: mock_response))

    response = api_instance.initiate_mpesa_stk_push(100, '254700000000', 'channel_id', 'provider', 'external_reference', 'customer_name', 'callback_url')
    assert response == mock_response

def test_fetch_transactions(api_instance, mocker):
    """Test the fetch_transactions method."""
    mock_response = {
        "transactions": []
    }
    mocker.patch('requests.get', return_value=mocker.Mock(status_code=200, json=lambda: mock_response))

    response = api_instance.fetch_transactions(1, 10)
    assert response == mock_response

def test_fetch_transaction_status(api_instance, mocker):
    """Test the fetch_transaction_status method."""
    mock_response = {
        "status": "completed"
    }
    mocker.patch('requests.get', return_value=mocker.Mock(status_code=200, json=lambda: mock_response))

    response = api_instance.fetch_transaction_status('reference_id')
    assert response == mock_response



from setuptools import setup, find_packages

setup(
    name="py-payhero",
    version="0.0.1",
    description="A Python SDK for interacting with the PayHero API",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Njeru Mtwaiti",
    author_email="newerbandit@proton.me",
    url="https://github.com/njeru-codes/PayHero-API-SDK-",
    license="MIT",
    packages=find_packages(),
    install_requires=[
        "requests"
    ],
    python_requires=">=3.6",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)